package cn.com.qmth.task.api.provider;

import io.swagger.annotations.ApiOperation;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import cn.com.qmth.task.base.QuartzManager;
import cn.com.qmth.task.base.ScheduleJob;
import cn.com.qmth.task.base.enums.JobStatus;
import cn.com.qmth.task.entity.ScheduleJobEntity;
import cn.com.qmth.task.service.ScheduleJobEntityService;

@RestController
@RequestMapping("${app.api.task}/taskScheduleApi")
public class TaskScheduleApi {
	
	@Autowired
	private ScheduleJobEntityService scheduleJobEntityService;
	
	@Autowired
	private QuartzManager quartzManager;
	
	@ApiOperation(value="启动job")
	@PostMapping("/executeJob/{jobName}")
	public void executeJob(@PathVariable String jobName){
		ScheduleJobEntity scheduleJobEntity = scheduleJobEntityService.findByJobName(jobName);
		ScheduleJob scheduleJob = scheduleJobEntityService.transformationToScheduleJob(scheduleJobEntity);
		try{
			quartzManager.addJob(scheduleJob);
			scheduleJobEntity.setJobStatus(JobStatus.RUNNING.name());
			scheduleJobEntity.setOperateResult(JobStatus.RUNNING.name()+" SUCCESS");
			scheduleJobEntityService.saveScheduleJobEntity(scheduleJobEntity);
		}catch(Exception e){
			scheduleJobEntity.setOperateResult(JobStatus.RUNNING.name()+" FAILED");
			scheduleJobEntityService.saveScheduleJobEntity(scheduleJobEntity);
			e.printStackTrace();
		}
	}
	
	@ApiOperation(value="暂停job")
	@PostMapping("/pauseJob/{jobName}")
	public void pauseJob(@PathVariable String jobName){
		ScheduleJobEntity scheduleJobEntity = scheduleJobEntityService.findByJobName(jobName);
		ScheduleJob scheduleJob = scheduleJobEntityService.transformationToScheduleJob(scheduleJobEntity);
		try{
			quartzManager.pauseJob(scheduleJob);
			scheduleJobEntity.setJobStatus(JobStatus.PAUSE.name());
			scheduleJobEntity.setOperateResult(JobStatus.PAUSE.name()+" SUCCESS");
			scheduleJobEntityService.saveScheduleJobEntity(scheduleJobEntity);
		}catch(Exception e){
			scheduleJobEntity.setOperateResult(JobStatus.PAUSE.name()+" FAILED");
			scheduleJobEntityService.saveScheduleJobEntity(scheduleJobEntity);
			e.printStackTrace();
		}
	}
	
	@ApiOperation(value="从暂停中恢复job")
	@PostMapping("/resumeJob/{jobName}")
	public void resumeJob(@PathVariable String jobName){
		ScheduleJobEntity scheduleJobEntity = scheduleJobEntityService.findByJobName(jobName);
		ScheduleJob scheduleJob = scheduleJobEntityService.transformationToScheduleJob(scheduleJobEntity);
		try{
			quartzManager.resumeJob(scheduleJob);
			scheduleJobEntity.setJobStatus(JobStatus.RESUME.name());
			scheduleJobEntity.setOperateResult(JobStatus.RESUME.name()+" SUCCESS");
			scheduleJobEntityService.saveScheduleJobEntity(scheduleJobEntity);
		}catch(Exception e){
			scheduleJobEntity.setOperateResult(JobStatus.RESUME.name()+" FAILED");
			scheduleJobEntityService.saveScheduleJobEntity(scheduleJobEntity);
			e.printStackTrace();
		}
	}
	
	@ApiOperation(value="删除job")
	@PostMapping("/deleteJob/{jobName}")
	public void deleteJob(@PathVariable String jobName){
		ScheduleJobEntity scheduleJobEntity = scheduleJobEntityService.findByJobName(jobName);
		ScheduleJob scheduleJob = scheduleJobEntityService.transformationToScheduleJob(scheduleJobEntity);
		try{
			quartzManager.deleteJob(scheduleJob);
			scheduleJobEntity.setJobStatus(JobStatus.DELETE.name());
			scheduleJobEntity.setOperateResult(JobStatus.DELETE.name()+" SUCCESS");
			scheduleJobEntityService.saveScheduleJobEntity(scheduleJobEntity);
		}catch(Exception e){
			scheduleJobEntity.setOperateResult(JobStatus.DELETE.name()+" FAILED");
			scheduleJobEntityService.saveScheduleJobEntity(scheduleJobEntity);
			e.printStackTrace();
		}
	}
	
}
